#include <linux/version.h>

#include <linux/string.h>
#include <linux/errno.h>
#include <linux/module.h>

#include "nikal.h"

#ifdef nNIKAL100_kCopyrightString
   MODULE_LICENSE(nNIKAL100_kCopyrightString);
#endif

#ifdef nNIKAL100_kAuthorString
   MODULE_AUTHOR(nNIKAL100_kAuthorString);
#endif

#ifdef nNIKAL100_kDescriptionString
   MODULE_DESCRIPTION(nNIKAL100_kDescriptionString);
#endif


#include <linux/pci.h>
#include <linux/mod_devicetable.h>

int nNIAPAL200_driverEntry(struct nNIKAL100_tDriver *driver);

static struct nNIKAL100_tDriver atomicDriver;
static void* nativeDriverRegistration;

/* native driver registration functions. These functions are auto-generated */
void *registerNativeDriver(nNIKAL100_tDriver*);
void unregisterNativeDriver(void*);

int init_module(void)
{
   atomicDriver.module = THIS_MODULE;

   if (nNIAPAL200_driverEntry(&atomicDriver))
      return -ENODEV;

   nativeDriverRegistration = registerNativeDriver(&atomicDriver);

   if (nativeDriverRegistration == NULL)
   {
      if (atomicDriver.unload)
         atomicDriver.unload(&atomicDriver);
      return -ENODEV;
   }

   return 0;
}

void cleanup_module(void)
{
   unregisterNativeDriver(nativeDriverRegistration);

   if (atomicDriver.unload)
      atomicDriver.unload(&atomicDriver);
}

static struct pci_device_id nipciidtable[] = {
   {0x1093, 0xC4C4, 0x1093, 0x7736, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7861, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x76CA, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7862, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x772B, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7853, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77FB, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x785B, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x772C, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7854, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77FC, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x785C, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x772D, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7855, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77FD, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x785D, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x772E, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7856, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78EF, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x772F, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7857, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77FE, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x785E, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7730, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7858, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x77FF, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x785F, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7731, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7859, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7800, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7860, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x7732, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x785A, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0xC4C4, 0x1093, 0x78F0, 0, 0, (unsigned long)&atomicDriver},
    { },
};
MODULE_DEVICE_TABLE(pci, nipciidtable);

static struct pci_driver niPciDriver = {
   .name = "niusrpriok",
   .id_table = nipciidtable,
};

void *registerNativeDriver(nNIKAL100_tDriver* ignore)
{
   return nNIKAL200_registerPCIDriver(&niPciDriver);
}
void unregisterNativeDriver(void* data)
{
   nNIKAL200_unregisterPCIDriver(data);
}


#define nNIKAL100_mExportSymbol EXPORT_SYMBOL
